@@ -1,5 +1,6 @@
 import base64
 import hmac
+import logging
 import time
 
 from ks3 import utils
@@ -17,6 +18,8 @@ except ImportError:
 
     sha256 = None
 
+logger = logging.getLogger(__name__)
+
 # query string argument of interest
 qsa_of_interest = ['acl', 'cors', 'defaultObjectAcl', 'location', 'logging',
                    'partNumber', 'policy', 'requestPayment', 'torrent',
@@ -29,7 +32,10 @@ qsa_of_interest = ['acl', 'cors', 'defaultObjectAcl', 'location', 'logging',
                    'adp', 'asyntask', 'querytask', 'domain',
                    'storageClass',
                    'websiteConfig',
-                   'compose', 'quota', 'policy', 'crr', 'fetch', 'append', 'mirror']
+                   'compose', 'quota', 'crr', 'fetch', 'append', 'position',
+                   'mirror', 'retention', 'recycle', 'recover', 'clear', 'inventory',
+                   'id', 'x-kss-process', 'encryption', 'accessmonitor', 'decompresspolicy',
+                   'migration', 'bucketqos', 'requesterqos']
 
 
 def url_encode(key):
@@ -39,8 +45,8 @@ def url_encode(key):
     # if '%20' in encode_key:
     #     encode_key = encode_key.replace('%20', '+')
 
-    if '%2A' in encode_key:
-        encode_key = encode_key.replace('%2A', '*')
+    # if '%2A' in encode_key:
+    #     encode_key = encode_key.replace('%2A', '*')
 
     if '%7E' in encode_key:
         encode_key = encode_key.replace('%7E', '~')
@@ -113,7 +119,9 @@ def canonical_headers(headers):
     return "\n".join(buf_list)
 
 
-def canonical_string(method, bucket="", key="", query_args=None, headers=None, expires=None):
+def canonical_string(method, bucket="", key="", query_args=None, headers=None, expires=None, policy=None):
+    if policy:
+        return policy_canonical_string(bucket, expires, policy)
     if not headers:
         headers = {}
     if not query_args:
@@ -142,8 +150,16 @@ def canonical_string(method, bucket="", key="", query_args=None, headers=None, e
         sign_list.append(can_headers)
     sign_list.append(can_resource)
     sign_str = "\n".join(sign_list)
+    logger.debug('sign_list: {0}'.format(sign_list))
     return sign_str
 
+def format_policy_param(policy):
+    return "X-Kss-Policy={0}".format(policy)
+
+def policy_canonical_string(bucket, expires, policy):
+    sign_str = "{0}\n/{1}/?{2}".format(expires, bucket, format_policy_param(policy))
+    logger.debug('sign_str: {0}'.format(sign_str))
+    return sign_str
 
 def encode(secret_access_key, str_to_encode, urlencode=False):
     secret_access_key = secret_access_key.encode('utf-8')
@@ -159,8 +175,11 @@ def add_auth_header(access_key_id, secret_access_key, headers, method, bucket, k
     if not access_key_id:
         return
     if 'Date' not in headers:
-        headers['Date'] = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())
+        from email.utils import formatdate
+        headers['Date'] = formatdate(time.time(), usegmt=True)
 
     c_string = canonical_string(method, bucket, key, query_args, headers)
+    c_string_encoded = encode(secret_access_key, c_string)
+    logger.debug('c_string_encoded: {0}'.format(c_string_encoded))
     headers['Authorization'] = \
-        "%s %s:%s" % ("KSS", access_key_id, encode(secret_access_key, c_string))
+        "%s %s:%s" % ("KSS", access_key_id, c_string_encoded)
