@@ -1,31 +1,33 @@
-
 import json
 import xml.sax
 
-
 from ks3 import handler
 
+
 class KS3ClientError(Exception):
     """
     General KS3 Client error
     """
-    def __init__(self, reason, *args):
+
+    def __init__(self, reason, request_id=None, *args):
         super(KS3ClientError, self).__init__(reason, *args)
         self.reason = reason
+        self.request_id = request_id
 
     def __repr__(self):
-        return 'KS3ClientError: %s' % self.reason
+        return 'KS3ClientError: %s, request_id: %s' % (self.reason, self.request_id)
 
     def __str__(self):
-        return 'KS3ClientError: %s' % self.reason
+        return repr(self)
+
 
 class KS3ServerError(Exception):
-    def __init__(self, status, reason, body=None, *args):
-        super(KS3ServerError, self).__init__(status, reason, body, *args)
+    def __init__(self, status, reason, body=None, request_id=None, *args):
+        super(KS3ServerError, self).__init__(status, reason, body, request_id, *args)
         self.status = status
         self.reason = reason
         self.body = body or ''
-        self.request_id = None
+        self.request_id = request_id
         self.error_code = None
         self._error_message = None
         self.message = ''
@@ -78,6 +80,8 @@ class KS3ServerError(Exception):
                         # that are just text strings that we want to preserve.
                         self.message = self.body
                         self.body = None
+        else:
+            self.message = 'No body in response.'
 
     def __getattr__(self, name):
         if name == 'error_message':
@@ -94,11 +98,11 @@ class KS3ServerError(Exception):
 
     def __repr__(self):
         return '%s: %s %s\n%s' % (self.__class__.__name__,
-                                  self.status, self.reason, self.body)
+                                  self.status, self.reason, self.body if self.body else self.request_id)
 
     def __str__(self):
         return '%s: %s %s\n%s' % (self.__class__.__name__,
-                                  self.status, self.reason, self.body)
+                                  self.status, self.reason, self.body if self.body else self.request_id)
 
     def startElement(self, name, attrs, connection):
         pass
@@ -120,13 +124,15 @@ class KS3ServerError(Exception):
         self.message = None
         self.box_usage = None
 
+
 class StorageResponseError(KS3ServerError):
     """
     Error in response from a storage service.
     """
-    def __init__(self, status, reason, body=None):
+
+    def __init__(self, status, reason, body=None, request_id=None):
         self.resource = None
-        super(StorageResponseError, self).__init__(status, reason, body)
+        super(StorageResponseError, self).__init__(status, reason, body, request_id=request_id)
 
     def startElement(self, name, attrs, connection):
         return super(StorageResponseError, self).startElement(
@@ -144,24 +150,30 @@ class StorageResponseError(KS3ServerError):
         for p in ('resource'):
             setattr(self, p, None)
 
+
 class S3ResponseError(StorageResponseError):
     """
     Error in response from S3.
     """
-    def __init__(self, status, reason, body=None):
-        super(S3ResponseError, self).__init__(status, reason, body)
+
+    def __init__(self, status, reason, body=None, request_id=None):
+        super(S3ResponseError, self).__init__(status, reason, body, request_id=request_id)
+
 
 class KS3ResponseError(S3ResponseError):
     """
     Error in response from KS3.
     """
+
     def __init__(self, status, reason, body=None):
         super(KS3ResponseError, self).__init__(status, reason, body)
 
+
 class StorageCreateError(KS3ServerError):
     """
     Error creating a bucket or key on a storage service.
     """
+
     def __init__(self, status, reason, body=None):
         self.bucket = None
         super(StorageCreateError, self).__init__(status, reason, body)
@@ -172,42 +184,49 @@ class StorageCreateError(KS3ServerError):
         else:
             return super(StorageCreateError, self).endElement(name, value, connection)
 
+
 class StoragePermissionsError(KS3ClientError):
     """
     Permissions error when accessing a bucket or key on a storage service.
     """
     pass
 
+
 class S3CreateError(StorageCreateError):
     """
     Error creating a bucket or key on S3.
     """
     pass
 
+
 class S3PermissionsError(StoragePermissionsError):
     """
     Permissions error when accessing a bucket or key on S3.
     """
     pass
 
+
 class StorageDataError(KS3ClientError):
     """
     Error receiving data from a storage service.
     """
     pass
 
+
 class S3DataError(StorageDataError):
     """
     Error receiving data from S3.
     """
     pass
 
+
 class StorageCopyError(KS3ClientError):
     """
     Error copying a key on a storage service.
     """
     pass
 
+
 class S3CopyError(StorageCopyError):
     """
     Error copying a key on S3.
@@ -219,6 +238,7 @@ class PleaseRetryException(Exception):
     """
     Indicates a request should be retried.
     """
+
     def __init__(self, message, response=None):
         self.message = message
         self.response = response
@@ -228,3 +248,9 @@ class PleaseRetryException(Exception):
             self.message,
             self.response
         )
+
+
+class ParamValidationError(Exception):
+    def __init__(self, msg):
+        self.msg = msg
+        super().__init__(self.msg)
