@@ -1,12 +1,16 @@
-from ks3.objectTagging import Tag
+from ks3.tagging import Tag
+from ks3.responseResult import ResponseMetadata
+
 
 class BucketLifecycle(object):
-    def __init__(self, rule=None):
+    def __init__(self, rule=None, *args, **kwargs):
         if rule is None:
             self.rule = []
         else:
             self.rule = rule
 
+        self.response_metadata = ResponseMetadata(**kwargs)
+
     def __repr__(self):
         return "rules: " + str(len(self.rule))
 
@@ -31,13 +35,14 @@ class BucketLifecycle(object):
 class Rule(object):
     def __init__(self, id=None, filter=None, status=None, expiration=None, transitions=None,
                  noncurrent_version_expiration=None,
-                 noncurrent_version_transition=None):
+                 noncurrent_version_transition=None, abort_incomplete_multipart_upload=None):
         self.id = id
         self.filter = filter
         self.status = status
         self.expiration = expiration
-        self.noncurrent_version_expiration=noncurrent_version_expiration
-        self.noncurrent_version_transition=noncurrent_version_transition
+        self.noncurrent_version_expiration = noncurrent_version_expiration
+        self.noncurrent_version_transition = noncurrent_version_transition
+        self.abort_incomplete_multipart_upload = abort_incomplete_multipart_upload
         if transitions is None:
             self.transitions = []
         else:
@@ -53,6 +58,8 @@ class Rule(object):
         if name == 'Transition':
             self.transitions.append(Transition())
             return self.transitions[-1]
+        if name == 'AbortIncompleteMultipartUpload':
+            self.abort_incomplete_multipart_upload = AbortIncompleteMultipartUpload()
         if name == 'NoncurrentVersionTransition':
             self.noncurrent_version_transition = NoncurrentVersionTransition()
             return self.noncurrent_version_transition
@@ -82,6 +89,8 @@ class Rule(object):
         if self.transitions is not None:
             for t in self.transitions:
                 s += t.to_xml()
+        if self.abort_incomplete_multipart_upload is not None:
+            s += self.abort_incomplete_multipart_upload.to_xml()
         if self.noncurrent_version_expiration is not None:
             s += self.noncurrent_version_expiration.to_xml()
         if self.noncurrent_version_transition is not None:
@@ -127,10 +136,12 @@ class Filter(object):
 
 
 class Transition(object):
-    def __init__(self, days=None, date=None, storage_class=None):
+    def __init__(self, days=None, date=None, storage_class=None, is_access_time=None, return_to_std_when_visit=None):
         self.days = days
         self.storage_class = storage_class
         self.date = date
+        self.is_access_time = is_access_time
+        self.return_to_std_when_visit = return_to_std_when_visit
 
     def startElement(self, name, attrs, connection):
         return None
@@ -142,6 +153,10 @@ class Transition(object):
             self.storage_class = value
         elif name == 'Date':
             self.date = value
+        elif name == 'IsAccessTime':
+            self.is_access_time = value
+        elif name == 'ReturnToStdWhenVisit':
+            self.return_to_std_when_visit = value
         else:
             setattr(self, name, value)
 
@@ -153,6 +168,10 @@ class Transition(object):
             s += '<Date>%s</Date>' % self.date
         if self.storage_class is not None:
             s += '<StorageClass>%s</StorageClass>' % self.storage_class
+        if self.is_access_time is not None:
+            s += '<IsAccessTime>%s</IsAccessTime>' % self.is_access_time
+        if self.return_to_std_when_visit is not None:
+            s += '<ReturnToStdWhenVisit>%s</ReturnToStdWhenVisit>' % self.return_to_std_when_visit
         s += '</Transition>'
         return s
 
@@ -208,6 +227,7 @@ class NoncurrentVersionTransition(object):
         s += '</NoncurrentVersionTransition>'
         return s
 
+
 class NoncurrentVersionExpiration(object):
     def __init__(self, noncurrent_days=None):
         self.noncurrent_days = noncurrent_days
@@ -218,3 +238,29 @@ class NoncurrentVersionExpiration(object):
             s += '<NoncurrentDays>%s</NoncurrentDays>' % self.noncurrent_days
         s += '</NoncurrentVersionExpiration>'
         return s
+
+
+class AbortIncompleteMultipartUpload(object):
+    def __init__(self, days_after_initiation=None, date=None):
+        self.days_after_initiation = days_after_initiation
+        self.date = date
+
+    def startElement(self, name, attrs, connection):
+        return None
+
+    def endElement(self, name, value, connection):
+        if name == 'DaysAfterInitiation':
+            self.days_after_initiation = value
+        elif name == 'Date':
+            self.date = value
+        else:
+            setattr(self, name, value)
+
+    def to_xml(self):
+        s = '<AbortIncompleteMultipartUpload>'
+        if self.days_after_initiation is not None:
+            s += '<DaysAfterInitiation>%s</DaysAfterInitiation>' % self.days_after_initiation
+        if self.date is not None:
+            s += '<Date>%s</Date>' % self.date
+        s += '</AbortIncompleteMultipartUpload>'
+        return s
