@@ -1,7 +1,12 @@
+from ks3.responseResult import ResponseMetadata
+
+
 class BucketQuota(object):
-    def __init__(self, quota=None):
+    def __init__(self, quota=None, *args, **kwargs):
         self.quota = quota
 
+        self.response_metadata = ResponseMetadata(**kwargs)
+
     def __repr__(self):
         return "quota "+self.quota
 
